//
//  ApplicationPanel.java
//  ApplicationPanel
//
//  Created by Abdelatif Nouqrat on Tue Apr 27 2004.
//  Copyright (c) 2004 ilink GmbH. All rights reserved.
//
package de.ilink.jtapi.makecall.ui;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import de.ilink.jtapi.makecall.Constants;


/**
 * This panel contains a JTextField and a JButton.
 *
 * @author Abdelatif Nouqrat
 */
public class ApplicationPanel extends JPanel implements PropertyChangeListener
{
	private ActionEventSupport fSupport = new ActionEventSupport();
	private DialButton fDialButton;
	private CallTextField fTextField;
	
	public ApplicationPanel()
	{
		setLayout(new FlowLayout(FlowLayout.LEFT));
		// action listener
		ActionListener actionListener = new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
			{			
                fSupport.fireActionEvent(e);
			}
        };
		// textfield
		fTextField = new CallTextField(actionListener);
		fTextField.setEnabled(false);
		add(fTextField);
		
		// button
		fDialButton = new DialButton(actionListener);
		fDialButton.setEnabled(false);
		add(fDialButton);
	}
	
	public void addActionListener(ActionListener l)
    {
        fSupport.addActionListener(l);
    }
    
    public void removeActionListener(ActionListener l)
    {
        fSupport.removeActionListener(l);
    }
	
	public void propertyChange (PropertyChangeEvent e)
	{		
		if (e.getPropertyName().equals(Constants.PROPERTY_NAME_TALKING))
		{
			fDialButton.setText("Drop");
			fDialButton.setActionCommand(Constants.DROPBUTTON_CLICKED);
			
			fTextField.setEnabled(false);
		}
		else if (e.getPropertyName().equals(Constants.PROPERTY_NAME_DROPPED))
		{
			fDialButton.setText("Dial");
			fDialButton.setActionCommand(Constants.DIALBUTTON_CLICKED);
			
			fTextField.setEnabled(true);
		}
		else if (e.getPropertyName().equals(Constants.PROPERTY_NAME_PROVIDER_IN_SERVICE))
		{
			fDialButton.setEnabled(true);
			fTextField.setEnabled(true);
		}
    }
}

class ActionEventSupport
{
    private java.util.Vector listenerList;
	
    public ActionEventSupport()
	{
        listenerList = new java.util.Vector();
    }
	
    public synchronized void addActionListener(ActionListener l)
    {
        if(!listenerList.contains(l))
        {
            listenerList.add(l);
        }
    }
    
    public synchronized void removeActionListener(ActionListener l)
    {
        if(listenerList.contains(l))
        {
            listenerList.remove(l);
        }
    }
    
    public void fireActionEvent(ActionEvent e)
    {
        java.util.Vector listCopy;
		
        synchronized(this)
        {
            listCopy = (java.util.Vector)listenerList.clone();
            for( java.util.Iterator i=listCopy.iterator(); i.hasNext();)
            {
                ((ActionListener) i.next()).actionPerformed(e);
            }
        }
    }
}


/**
 * A button with a special action command and an action listener.
 */
class DialButton extends JButton
{
	public DialButton(ActionListener anActionListener)
	{
		super("Dial");
		setActionCommand(Constants.DIALBUTTON_CLICKED);
		addActionListener(anActionListener);
	}
}

/**
 * A textfield with a special action command and a doc listener.
 */
class CallTextField extends JTextField
{
	private boolean fIsDocumentUpdate = false;
	
	public CallTextField(ActionListener anActionListener)
	{
		super("");
		setPreferredSize(new Dimension(400,20));
		setActionCommand(Constants.TEXTFIELD_ENTER);
		addActionListener(anActionListener);
		
		// document listener
		DocumentListener docListener = new DocumentListener()
        {
			public void changedUpdate(DocumentEvent de)
			{
			}
			
			public void insertUpdate(DocumentEvent de)
			{
				String dialNumber = getText();
				setActionCommand(dialNumber);
				fIsDocumentUpdate = true;
				fireActionPerformed();
			}
			
			public void removeUpdate(DocumentEvent de)
			{
				String dialNumber = getText();
				setActionCommand(dialNumber);
				fIsDocumentUpdate = true;
				fireActionPerformed();
			}
        };
		
		getDocument().addDocumentListener(docListener);
	}
	
	
	protected void fireActionPerformed()
	{
		if (!fIsDocumentUpdate)
			setActionCommand(Constants.DIALBUTTON_CLICKED); // should dial if we press enter...
		
		fIsDocumentUpdate = false;
		super.fireActionPerformed();
	}	
}
