package de.ilink.examples.ilink;

import java.io.FileInputStream;
import javax.telephony.*;
import javax.telephony.events.*;
import javax.swing.*;

import de.ilink.cti.util.XMLPropertyParser;
import de.ilink.examples.BLF;

/**
 * Demo application that displays the busy lines ("BelegtLampenFeld") of the phones
 * given in the "BLFPhones.xml" file. 
 * The "BLFPhones.xml" config file is structured as follows :
 * <!--
 * <ul>
 * <li><b>monitoredAddresses : root element</li>
 * 
 * <li><b>serverConnectionInfo</b>: Connection info for the creation of the provider.</li>
 * <li><b>address</b>: contains following information </li>
 * <ul>
 * <li><b>name</b>: name of the phone user for the BLF.</li>
 * <li><b>phonenumber</b>: monitored device for the BLF.</li>
 * </ul>
 * </ul>
 * -->
 * @author Abdeltif.Nouqrat@ilink.de
 * @version $Id: BLFMain.java,v 1.5 2004/05/19 13:27:23 nouqrat Exp $
 */

public class BLFMain implements ProviderObserver {
	private Provider fProvider = null;
	private XMLPropertyParser fPhoneConfig;
	private String fConnectInfo;
	private BLF fBLF;

	BLFMain() {
		parseConfigFile("BLFPhones.xml");
		JtapiPeer peer = null;
		try {
			peer =
				JtapiPeerFactory.getJtapiPeer(
					"de.ilink.cti.cstajtapi.JtapiPeerImpl");
		} catch (JtapiPeerUnavailableException ex) {
			JOptionPane.showMessageDialog(
				null,
				"Peer is unavailable \n please check your configuration " 
				+ "on the BLFPhones.xml file and try again",
				"unavailable peer",
				JOptionPane.WARNING_MESSAGE);
			ex.printStackTrace();
			System.exit(0);
		}
		try {
			fProvider = peer.getProvider(fConnectInfo);
		} catch (ProviderUnavailableException ex) {
			JOptionPane.showMessageDialog(
				null,
				"Provider is unavailable \n please check your configuration ",
				"busy Provider",
				JOptionPane.WARNING_MESSAGE);
			ex.printStackTrace();
			System.exit(-1);
		}

		try {
			fProvider.addObserver(this);
		} catch (Exception ex) {
			JOptionPane.showMessageDialog(
				null,
				"Cannot add Provider. Is unavailable !",
				"Cannot add Provider observer (" + ex.getMessage(),
				JOptionPane.WARNING_MESSAGE);
			ex.printStackTrace();
			System.exit(-1);
		}
		fBLF = new BLF(fProvider);
	}

	/**
	 * parseConfigFile() parses the following information
	 * on the given configuration file.
	 * serverConnectionInfo connection information for the creation of the provider
	 * @param fileName the config file to be parsed
	 */
	private void parseConfigFile(String fileName) {
		try {
			//java.io.FileNotFoundException
			FileInputStream fis = new FileInputStream("BLFPhones.xml");
			//java.io.IOException
			byte data[] = new byte[fis.available()];
			fis.read(data);
			fis.close();
			//de.ilink.cti.cstaxml.CSTAException;
			fPhoneConfig =
				XMLPropertyParser.createPropertyParserFromXMLString(
					new String(data));
			fConnectInfo =
				fPhoneConfig.getNodeValueForKey(
					"monitoredAddresses.serverConnectionInfo");
		} catch (Exception ex) {
			ex.printStackTrace();
			System.exit(-1);
		}
	}

	/**
	 * createBLF() parses the information needed to be displayed 
	 * and adds these information to the GUI's panel
	 * phoneName : phoneuser name. Will be added as a label 
	 * phoneNumber : phone number to the user. Will be added as a button
	 */
	private void createBLF() {
		int arrSize =
			fPhoneConfig.getArraySizeForNode("monitoredAddresses", "address");
		/* a loop for reading the property values: 
		 * <name> and <phonenumber> 
		 * these values are to be added into a BLF-instance
		 * a name as a Label, a phoneNumber as a buttontext 
		 */
		for (int i = 0; i < arrSize; i++) {
			String phoneName =
				fPhoneConfig.getNodeValueForKey(
					"monitoredAddresses.address[" + i + "].name");
			String phoneNumber =
				fPhoneConfig.getNodeValueForKey(
					"monitoredAddresses.address[" + i + "].phonenumber");
			fBLF.addAddress(phoneName, phoneNumber);

		}
		fBLF.pack();
		fBLF.setVisible(true);

	}

	/**
	 * providerChangedEvent() shows all provider events  
	 * @see javax.telephony.ProviderObserver#
	 * providerChangedEvent(javax.telephony.events.ProvEv[])
	 */
	public void providerChangedEvent(ProvEv[] aProviderEventArr) {
		for (int i = 0; i < aProviderEventArr.length; i++) {
			System.out.println(
				"providerChangedEvent: "
					+ aProviderEventArr[i].getClass().getName());
		}
	}

	/**
	 * main method creates a BLFMain instance 
	 * and builds the GUI of the BLF application
	 * @param args
	 */
	public static void main(String args[]) {
		BLFMain theApp = new BLFMain();
		theApp.createBLF();
	}

	/**
	 * getProvider() returns the provider
	 * @return fProvider the provider created using
	 * the parsed serverConnection information
	 */
	public Provider getProvider() {
		return fProvider;
	}
}